#/***********************************************************************/
#/*  This file is part of the uVision/ARM development tools             */
#/*  Copyright KEIL ELEKTRONIK GmbH 2002-2006                           */
#/***********************************************************************/
#/*                                                                     */
#/*  ADUC706x.S:  Startup file ADI ADuC706x device series               */
#/*                                                                     */
#/***********************************************************************/

.include "ADuC7060.equ"
/* 
//*** <<< Use Configuration Wizard in Context Menu >>> *** 
*/


# *** Startup Code (executed after Reset) ***
                

# Standard definitions of Mode bits and Interrupt (I & F) flags in PSRs 
 
        .equ    Mode_USR,       0x10
        .equ    Mode_FIQ,       0x11
        .equ    Mode_IRQ,       0x12
        .equ    Mode_SVC,       0x13
        .equ    Mode_ABT,       0x17
        .equ    Mode_UND,       0x1B
        .equ    Mode_SYS,       0x1F

        .equ    I_Bit,          0x80    /* when I bit is set, IRQ is disabled */
        .equ    F_Bit,          0x40    /* when F bit is set, FIQ is disabled */


/*
// <h> Stack Configuration (Stack Sizes in Bytes)
//   <o0> Undefined Mode      <0x0-0xFFFFFFFF:4>
//   <o1> Supervisor Mode     <0x0-0xFFFFFFFF:4>
//   <o2> Abort Mode          <0x0-0xFFFFFFFF:4>
//   <o3> Fast Interrupt Mode <0x0-0xFFFFFFFF:4>
//   <o4> Interrupt Mode      <0x0-0xFFFFFFFF:4>
//   <o5> User/System Mode    <0x0-0xFFFFFFFF:4>
// </h>
*/
        .equ    UND_Stack_Size, 0x00000080
        .equ    SVC_Stack_Size, 0x00000080
        .equ    ABT_Stack_Size, 0x00000080
        .equ    FIQ_Stack_Size, 0x00000100
        .equ    IRQ_Stack_Size, 0x00000100
        .equ    USR_Stack_Size, 0x00000400


/*
// <q> Enable Stdlib I/O 
// <i> enable this option when your code contains printf calls 
*/
        .equ    En_StdIO,       0


# Starupt Code must be linked first at Address at which it expects to run.

        .section "startup","x"
        .arm

        .global _startup
        .func   _startup
_startup:


# Exception Vectors
#  Mapped to Address 0.
#  Absolute addressing mode must be used.
#  Dummy Handlers are implemented as infinite loops which can be modified.

Vectors:        LDR     PC, Reset_Addr         
                LDR     PC, Undef_Addr
                LDR     PC, SWI_Addr
                LDR     PC, PAbt_Addr
                LDR     PC, DAbt_Addr
                NOP                            /* Reserved Vector */
                LDR     PC, IRQ_Addr
                LDR     PC, FIQ_Addr

Reset_Addr:     .word   Reset_Handler
Undef_Addr:     .word   ADI_UNDEF_Interrupt_Setup 
SWI_Addr:       .word   ADI_SWI_Interrupt_Setup 
PAbt_Addr:      .word   ADI_PABORT_Interrupt_Setup 
DAbt_Addr:      .word   ADI_DABORT_Interrupt_Setup 
IRQ_Addr:       .word   ADI_IRQ_Nested_Interrupt_Setup 
FIQ_Addr:       .word   ADI_FIQ_Nested_Interrupt_Setup 

# Reset Handler

Reset_Handler:  


# Setup Stack for each mode 

                LDR     R0, =Top_Stack 
 
#  Enter Undefined Instruction Mode and set its Stack Pointer 
                MSR     CPSR_c, #Mode_UND|I_Bit|F_Bit
                MOV     SP, R0 
                SUB     R0, R0, #UND_Stack_Size

#  Enter Abort Mode and set its Stack Pointer 
                MSR     CPSR_c, #Mode_ABT|I_Bit|F_Bit 
                MOV     SP, R0 
                SUB     R0, R0, #ABT_Stack_Size 
 
#  Enter FIQ Mode and set its Stack Pointer 
                MSR     CPSR_c, #Mode_FIQ|I_Bit|F_Bit 
                MOV     SP, R0 
                SUB     R0, R0, #FIQ_Stack_Size 
 
#  Enter IRQ Mode and set its Stack Pointer 
                MSR     CPSR_c, #Mode_IRQ|I_Bit|F_Bit 
                MOV     SP, R0 
                SUB     R0, R0, #IRQ_Stack_Size 
 
#  Enter Supervisor Mode and set its Stack Pointer 
                MSR     CPSR_c, #Mode_SVC|I_Bit|F_Bit 
                MOV     SP, R0 
                SUB     R0, R0, #SVC_Stack_Size 
 
#  Enter User Mode and set its Stack Pointer 
                MSR     CPSR_c, #Mode_USR 
                MOV     SP, R0 
 
#  Setup a default Stack Limit (when compiled with "-mapcs-stack-check")
                SUB     SL, SP, #USR_Stack_Size


# Relocate .data section (Copy from ROM to RAM) 
                LDR     R1, =_etext 
                LDR     R2, =_data 
                LDR     R3, =_edata 
LoopRel:        CMP     R2, R3 
                LDRLO   R0, [R1], #4 
                STRLO   R0, [R2], #4 
                BLO     LoopRel 
 
 
# Clear .bss section (Zero init)                 
                MOV     R0, #0 
                LDR     R1, =__bss_start__ 
                LDR     R2, =__bss_end__ 
LoopZI:         CMP     R1, R2 
                STRLO   R0, [R1], #4 
                BLO     LoopZI 
 

# Enter the C code

.if En_StdIO
            
# Enter the standard system startup code required for stdlib I/O  
              B       _start    
                              
.else

                LDR     LR, =__Return_from_main
                LDR     R0, =main
                BX      R0

__Return_from_main:
                B       __Return_from_main

.endif


        .size   _startup, . - _startup 
        .endfunc 


        .text
        .arm

# Handle the Nested VIC (IRQ) on the ADUC7060 
        .global ADI_IRQ_Nested_Interrupt_Setup
        .func   ADI_IRQ_Nested_Interrupt_Setup
ADI_IRQ_Nested_Interrupt_Setup:

#======================== Interrupts disabled!=================================#
#========================     IRQ Mode!       =================================#
        # This routine consumes 28 bytes of irq stack per nested interrupt
        # => 224 (8 x 28) due to the 8 maximum interrupt nesting.

        # This routine consumes 8 bytes of SYS stack per nested interrupt
        # => 64 (8 x 8) due to the 8 maximum interrupt nesting.

        # R12 is banked in FIQ mode is handling of this reqister is different
        # between the IRQ and FIQ handlers

        # Stack modified registers (potentially callee modified too)
        STMDB   SP!    ,{R0-R3,R12,LR}

        # Stack the SPSR
        MRS     R0      ,SPSR
        STMDB   SP!    ,{R0}

        # Retrieve handler for this interrupt
        LDR     R0      ,=IRQVEC
        LDR     R0      ,[R0]
        LDR     R2      ,[R0]

        # Allow new higher priority IRQs (and FIQs if enabled) in system mode
        MRS     R0      ,CPSR
        BIC     R0      ,R0     ,#I_Bit
        ORR     R1      ,R0     ,#Mode_SYS
        MSR     CPSR_c  ,R1 

#======================== Interrupts enabled! =================================#
#========================     SYS Mode!       =================================#
        # We need to store the R12,LR of system mode
        STMDB   SP!    ,{R12,LR}

        # Calling interrupt handler in system mode with interrupts enabled
        MOV     LR,PC
        BX      R2

        # We need to restore the R12,LR of system ,mode also
        LDMIA   SP!    ,{R12,LR}

        # Change back to IRQ mode, disabling I interrupts 
        MRS     R0      ,CPSR
        BIC     R0      ,R0     ,#Mode_SYS
        ORR     R0      ,R0     ,#(Mode_IRQ | I_Bit)
        MSR     CPSR_c  ,R0 
#======================== Interrupts disabled!=================================#
#========================     IRQ Mode!       =================================#

        # Clear the highest priority interrupt by writing 0xFF to IRQSTAN
        MOV      R0     ,#0xFF     
        LDR      R1     ,=IRQSTAN
        STR      R0     ,[R1]

        # Restore the SPSR
        LDMIA   SP!     ,{R0}
        MSR     SPSR    ,R0

        # Restore modified registers 
        LDMIA   SP!    ,{R0-R3,R12,LR}

        # Return from interrupt
        SUBS    PC     ,LR,#0x0004
#======================== Interrupts enabled! =================================#
        .endfunc


        .text
        .arm

# Handle the Nested VIC (FIQ) on the ADUC7060 
        .global ADI_FIQ_Nested_Interrupt_Setup
        .func   ADI_FIQ_Nested_Interrupt_Setup
ADI_FIQ_Nested_Interrupt_Setup:

#======================== Interrupts disabled!=================================#
#========================     FIQ Mode!       =================================#
        # This routine consumes 28 bytes of irq stack per nested interrupt
        # => 224 (8 x 28) due to the 8 maximum interrupt nesting.

        # This routine consumes 8 bytes of SYS stack per nested interrupt
        # => 64 (8 x 8) due to the 8 maximum interrupt nesting.

        # R12 is banked in FIQ mode is handling of this reqister is different
        # between the IRQ and FIQ handlers

        # Stack modified registers (potentially callee modified too)
        STMDB   SP!    ,{R0-R3,LR}

        # Stack the SPSR
        MRS     R0      ,SPSR
        STMDB   SP!    ,{R0}

        # Retrieve handler for this interrupt
        LDR     R0      ,=FIQVEC
        LDR     R0      ,[R0]
        LDR     R2      ,[R0]

        # Allow new higher priority FIQs in system mode
        MRS     R0      ,CPSR
        BIC     R0      ,R0     ,#F_Bit
        ORR     R1      ,R0     ,#Mode_SYS
        MSR     CPSR_c  ,R1 


#======================== Interrupts enabled! =================================#
#========================     SYS Mode!       =================================#
        # We need to store the R12,LR of system mode also
        STMDB   SP!    ,{R12,LR}

        # Calling interrupt handler in system mode with interrupts enabled
        MOV     LR,PC
        BX      R2

        # We need to restore the R12,LR of system ,mode also
        LDMIA   SP!    ,{R12,LR}

        # Change back to FIQ mode, disabling F interrupts 
        MRS     R0      ,CPSR
        BIC     R0      ,R0     ,#Mode_SYS
        ORR     R0      ,R0     ,#(Mode_FIQ | F_Bit)
        MSR     CPSR_c  ,R0 
#======================== Interrupts disabled!=================================#
#========================     FIQ Mode!       =================================#

        # Clear the highest priority interrupt by writing 0xFF to FIQSTAN
        MOV      R0     ,#0xFF     
        LDR      R1     ,=FIQSTAN
        STR      R0     ,[R1]


        # Restore the SPSR
        LDMIA   SP!    ,{R0}
        MSR     SPSR    ,R0

        # Restore modified registers 
        LDMIA   SP!    ,{R0-R3,LR}

        # Return from interrupt
        SUBS    PC     ,LR,#0x0004
#======================== Interrupts enabled! =================================#
        .endfunc

        .end
